#include <ultra64.h>
#include "sm64.h"
#include "behavior_data.h"
#include "model_ids.h"
#include "seq_ids.h"
#include "dialog_ids.h"
#include "segment_symbols.h"
#include "level_commands.h"
#include "game/level_update.h"
#include "levels/scripts.h"
#include "actors/common1.h"
#include "make_const_nonconst.h"

#include "areas/1/custom.model.inc.h"

#include "levels/castle_grounds/header.h"
#include "levels/castle_inside/header.h"

extern u8 _castle_grounds_segment_ESegmentRomStart[];
extern u8 _castle_grounds_segment_ESegmentRomEnd[];

const LevelScript level_castle_grounds_entry[] = {
	INIT_LEVEL(),
	LOAD_MIO0(0x07, _castle_inside_segment_7SegmentRomStart, _castle_inside_segment_7SegmentRomEnd),
	LOAD_RAW(0x1A, _castle_insideSegmentRomStart, _castle_insideSegmentRomEnd),
	LOAD_RAW(0x0E, _castle_grounds_segment_ESegmentRomStart, _castle_grounds_segment_ESegmentRomEnd),
	LOAD_MIO0(0xA, _SkyboxCustom21131216_skybox_mio0SegmentRomStart, _SkyboxCustom21131216_skybox_mio0SegmentRomEnd),
	LOAD_MIO0(8, _common0_mio0SegmentRomStart, _common0_mio0SegmentRomEnd),
	LOAD_RAW(15, _common0_geoSegmentRomStart, _common0_geoSegmentRomEnd),
	LOAD_MIO0(5, _group8_mio0SegmentRomStart, _group8_mio0SegmentRomEnd),
	LOAD_RAW(12, _group8_geoSegmentRomStart, _group8_geoSegmentRomEnd),
	LOAD_MIO0(6, _group15_mio0SegmentRomStart, _group15_mio0SegmentRomEnd),
	LOAD_RAW(13, _group15_geoSegmentRomStart, _group15_geoSegmentRomEnd),
	ALLOC_LEVEL_POOL(),
	MARIO(/*model*/ MODEL_MARIO, /*behParam*/ 0x00000001, /*beh*/ bhvMario),
	// Level Specific Models
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_BOWSER_TRAP,        castle_geo_000F18),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_WATER_LEVEL_PILLAR, castle_geo_001940),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_CLOCK_MINUTE_HAND,  castle_geo_001530),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_CLOCK_HOUR_HAND,    castle_geo_001548),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_CLOCK_PENDULUM,     castle_geo_001518),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_CASTLE_DOOR,        castle_door_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_WOODEN_DOOR,        wooden_door_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_METAL_DOOR,         metal_door_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_CASTLE_DOOR_UNUSED, castle_door_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_WOODEN_DOOR_UNUSED, wooden_door_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_DOOR_0_STARS,       castle_door_0_star_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_DOOR_1_STAR,        castle_door_1_star_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_DOOR_3_STARS,       castle_door_3_stars_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_KEY_DOOR,           key_door_geo),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_STAR_DOOR_30_STARS, castle_geo_000F00),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_STAR_DOOR_8_STARS,  castle_geo_000F00),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_STAR_DOOR_50_STARS, castle_geo_000F00),
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_STAR_DOOR_70_STARS, castle_geo_000F00),
	// Unspecified Models
	LOAD_MODEL_FROM_GEO(22,  warp_pipe_geo),
	LOAD_MODEL_FROM_GEO(23,  bubbly_tree_geo),
	LOAD_MODEL_FROM_GEO(24,  spiky_tree_geo),
	LOAD_MODEL_FROM_GEO(25,  snow_tree_geo),
	LOAD_MODEL_FROM_GEO(27,  palm_tree_geo),
	LOAD_MODEL_FROM_GEO(28,  castle_door_geo),
	LOAD_MODEL_FROM_GEO(29,  wooden_door_geo),
	LOAD_MODEL_FROM_GEO(31,  metal_door_geo),
	LOAD_MODEL_FROM_GEO(32,  hazy_maze_door_geo),
	LOAD_MODEL_FROM_GEO(34,  castle_door_0_star_geo),
	LOAD_MODEL_FROM_GEO(35,  castle_door_1_star_geo),
	LOAD_MODEL_FROM_GEO(36,  castle_door_3_stars_geo),
	LOAD_MODEL_FROM_GEO(37,  key_door_geo),
	LOAD_MODEL_FROM_GEO(38,  castle_door_geo),
	LOAD_MODEL_FROM_GEO(39,  wooden_door_geo),
	LOAD_MODEL_FROM_GEO(41,  metal_door_geo),
	// LOAD_MODEL_FROM_DL(84, 0x05002e00, 4),
	// LOAD_MODEL_FROM_DL(86, 0x05003120, 4),
	// LOAD_MODEL_FROM_DL(132, 0x08025f08, 4),
	// LOAD_MODEL_FROM_DL(158, 0x0302c8a0, 4),
	// LOAD_MODEL_FROM_DL(159, 0x0302bcd0, 4),
	// LOAD_MODEL_FROM_DL(161, 0x0301cb00, 4),
	// LOAD_MODEL_FROM_DL(164, 0x04032a18, 4),
	// LOAD_MODEL_FROM_DL(201, 0x080048e0, 4),
	// LOAD_MODEL_FROM_DL(218, 0x08024bb8, 4),
	JUMP_LINK(script_func_global_1),
	JUMP_LINK(script_func_global_9),
	JUMP_LINK(script_func_global_16),
	JUMP_LINK(local_area_castle_grounds_1_),
	FREE_LEVEL_POOL(),
	MARIO_POS(1, 0, 32, 65, 1037),
	CALL(/*arg*/ 0, /*func*/ lvl_init_or_update),
	CALL_LOOP(/*arg*/ 1, /*func*/ lvl_init_or_update),
	CLEAR_LEVEL(),
	SLEEP_BEFORE_EXIT(/*frames*/ 1),
	EXIT(),
};

const LevelScript local_area_castle_grounds_1_[] = {
	AREA(1, Geo_castle_grounds_1_0x144cf00),
	TERRAIN(col_castle_grounds_1_0xe05c690),
	SET_BACKGROUND_MUSIC(0, 5),
	TERRAIN_TYPE(0),
	JUMP_LINK(local_objects_castle_grounds_1_),
	JUMP_LINK(local_warps_castle_grounds_1_),
	END_AREA(),
	RETURN()
};

const LevelScript local_objects_castle_grounds_1_[] = {
	OBJECT_WITH_ACTS(0, 2397, 83, 2490, 0, 270, 0, 0x370000,  bhvLaunchStarCollectWarp, 31),
	OBJECT_WITH_ACTS(0, 2448, 56, 1944, 0, 195, 0, 0x690000,  bhvDeathWarp, 31),
	OBJECT_WITH_ACTS(0, 2485, -205, 2167, 0, 0, 0, 0x140000,  bhvWarp, 31),
	OBJECT_WITH_ACTS(34, -2486, 307, -2542, 0, 0, 0, 0x5000000,  bhvDoor, 31),
	OBJECT_WITH_ACTS(0, -2488, 400, -2792, 0, 0, 0, 0x1c0000,  bhvWarp, 31),
	OBJECT_WITH_ACTS(0, -2480, 1, -1506, 0, 180, 0, 0x340000,  bhvLaunchStarCollectWarp, 31),
	OBJECT_WITH_ACTS(0, 2784, 175, -1856, 0, 268, 0, 0xa4b0000,  bhvWarp, 31),
	OBJECT_WITH_ACTS(0, 2640, -214, 2166, 0, 0, 0, 0x140000,  bhvWarp, 31),
	OBJECT_WITH_ACTS(0, 0, 0, 0, 0, 0, 0, 0x330000,  bhvLaunchStarCollectWarp, 31),
	OBJECT_WITH_ACTS(0, 0, 0, 0, 0, 0, 0, 0x650000,  bhvDeathWarp, 31),
	OBJECT_WITH_ACTS(0, 285, 37, 292, 0, 0, 0, 0x1f0000,  bhvInstantActiveWarp, 31),
	OBJECT_WITH_ACTS(0, -2491, 0, -1409, 0, 0, 0, 0x660000,  bhvDeathWarp, 31),
	RETURN()
};

const LevelScript local_warps_castle_grounds_1_[] = {
	WARP_NODE(50, 6, 1, 50, 0),
	WARP_NODE(100, 6, 1, 100, 0),
	WARP_NODE(20, 9, 1, 20, 0),
	WARP_NODE(75, 6, 1, 75, 0),
	WARP_NODE(29, 18, 1, 29, 0),
	WARP_NODE(48, 12, 1, 48, 0),
	WARP_NODE(52, 16, 1, 52, 0),
	WARP_NODE(102, 16, 1, 102, 0),
	WARP_NODE(28, 24, 1, 28, 0),
	WARP_NODE(55, 16, 1, 55, 0),
	WARP_NODE(105, 16, 1, 105, 0),
	WARP_NODE(240, 6, 1, 50, 0),
	WARP_NODE(241, 6, 1, 100, 0),
	RETURN()
};